@@ -1,6 +1,17 @@
 # Liblouis Python ctypes bindings
 #
-# Copyright (C) 2009, 2010 James Teh <jamie@jantrid.net>
+# Copyright (C) 2009-2010 James Teh <jamie@jantrid.net>
+# Copyright (C) 2010-2012 Michael Whapples <mwhapples@gmail.com>
+# Copyright (C) 2012-2024 Christian Egli <christian.egli@sbs.ch>
+# Copyright (C) 2015-2016 Davy Kager <mail@davykager.nl>
+# Copyright (C) 2016 matt venn <matt@mattvenn.net>
+# Copyright (C) 2018-2025 Leonard de Ruijter <alderuijter@gmail.com>
+# Copyright (C) 2018 Bue Vester-Andersen <bue@vester-andersen.dk>
+# Copyright (C) 2019 Dave Mielke <dave@mielke.cc>
+# Copyright (C) 2019-2020 André-Abush Clause <dev@andreabc.net>
+# Copyright (C) 2019 Bert Frees <bertfrees@gmail.com>
+# Copyright (C) 2023 Rob Beezer <beezer@ups.edu>
+# Copyright (C) 2024 Daniel Garcia Moreno <daniel.garcia@suse.com>
 #
 # This file is part of liblouis.
 #
@@ -29,35 +40,36 @@ a list of translation tables to use. Please see the liblouis documentation
 concerning the C{tableList} parameter to the C{lou_translateString}
 function for information about how liblouis searches for these tables.
 
-@author: Michael Curran <mick@kulgan.net>
-@author: James Teh <jamie@jantrid.net>
-@author: Eitan Isaacson <eitan@ascender.com>
-@author: Michael Whapples <mwhapples@aim.com>
-@author: Davy Kager <mail@davykager.nl>
-@author: Leonard de Ruijter <alderuijter@gmail.com>
-@author: Babbage B.V. <info@babbage.com>
-@author: Andre-Abush Clause <dev@andreabc.net>
+:author: Michael Curran <mick@kulgan.net>
+:author: James Teh <jamie@jantrid.net>
+:author: Eitan Isaacson <eitan@ascender.com>
+:author: Michael Whapples <mwhapples@aim.com>
+:author: Davy Kager <mail@davykager.nl>
+:author: Leonard de Ruijter <alderuijter@gmail.com>
+:author: Babbage B.V. <info@babbage.com>
+:author: Andre-Abush Clause <dev@andreabc.net>
 """
 
+from atexit import register
+from collections.abc import Generator, Iterable, Sequence
+from ctypes import (
+    Array,
+    CFUNCTYPE,
+    POINTER,
+    c_char,
+    c_char_p,
+    c_int,
+    c_ushort,
+    cdll,
+    create_string_buffer,
+    string_at,
+)
 from sys import (
     byteorder,
     getdefaultencoding,
     getfilesystemencoding,
     platform,
 )
-from atexit import register
-from ctypes import (
-    c_ushort,
-    CFUNCTYPE,
-    cdll,
-    c_char_p,
-    c_char,
-    c_int,
-    POINTER,
-    byref,
-    create_string_buffer,
-    string_at,
-)
 
 try:  # Native win32
     from ctypes import WINFUNCTYPE, windll
@@ -70,38 +82,45 @@ _is_windows = platform == "win32"
 _endianness = "be" if byteorder == "big" else "le"
 
 # { Module Configuration
-#: Specifies the charSize (in bytes) used by liblouis.
-#: This is fetched once using L{liblouis.lou_charSize}.
-#: Call it directly, since L{charSize} is not yet defined.
-#: @type: int
-wideCharBytes = liblouis.lou_charSize()
-#: Specifies the number by which the input length should be multiplied
-#: to calculate the maximum output length.
-#: @type: int
-# This default will handle the case where every input character is
-# undefined in the translation table.
-outlenMultiplier = 4 + wideCharBytes * 2
-#: Specifies the encoding to use when encode/decode file/dir name
-#: @type: str
-fileSystemEncoding = "mbcs" if _is_windows else getfilesystemencoding()
-#: Specifies the encoding to use when converting from byte strings to unicode strings.
-#: @type: str
-conversionEncoding = "utf_%d_%s" % (wideCharBytes * 8, _endianness)
-# }
+wideCharBytes: int = liblouis.lou_charSize()
+"""
+Specifies the charSize (in bytes) used by liblouis.
+This is fetched once using `liblouis.lou_charSize`.
+Call it directly, since `charSize` is not yet defined.
+"""
+outlenMultiplier: int = 4 + wideCharBytes * 2
+"""
+Specifies the number by which the input length should be multiplied
+to calculate the maximum output length.
+The default will handle the case where every input character is
+undefined in the translation table.
+"""
+fileSystemEncoding: str = "mbcs" if _is_windows else getfilesystemencoding()
+"""Specifies the encoding to use when encode/decode file/dir name."""
+conversionEncoding: str = "utf_%d_%s" % (wideCharBytes * 8, _endianness)
+"""Specifies the encoding to use when converting from byte strings to unicode strings."""
 
 
 # Some general utility functions
-def _createTablesString(tablesList):
+TableListT = list[str | bytes]
+
+
+def _createTableBuf(tablesList: TableListT) -> Array[c_char]:
     """Creates a tables string for liblouis calls"""
-    return b",".join(
-        [
-            x.encode(fileSystemEncoding) if isinstance(x, str) else bytes(x)
-            for x in tablesList
-        ]
+    return create_string_buffer(
+        b",".join(
+            [
+                x.encode(fileSystemEncoding) if isinstance(x, str) else bytes(x)
+                for x in tablesList
+            ]
+        )
     )
 
 
-def _createTypeformbuf(length, typeform=None):
+def _createTypeformbuf(
+    length: int,
+    typeform: Iterable[int] | None = None,
+) -> Array[c_ushort]:
     """Creates a typeform buffer for liblouis calls"""
     return (c_ushort * length)(*typeform) if typeform else (c_ushort * length)()
 
@@ -109,13 +128,15 @@ def _createTypeformbuf(length, typeform=None):
 ENCODING_ERROR_HANDLER = "surrogatepass"
 
 
-def createEncodedByteString(
-    x, encoding=conversionEncoding, errors=ENCODING_ERROR_HANDLER
-):
-    return str(x).encode(encoding, errors)
+def _createEncodedBuf(
+    x: str,
+    encoding: str = conversionEncoding,
+    errors: str = ENCODING_ERROR_HANDLER,
+) -> Array[c_char]:
+    return create_string_buffer(x.encode(encoding, errors))
 
 
-def _processTableFiles(stringArray):
+def _processTableFiles(stringArray: Sequence[bytes] | None) -> Generator[str]:
     if not stringArray:
         return
     i = 0
@@ -125,23 +146,28 @@ def _processTableFiles(stringArray):
     liblouis.lou_freeTableFiles(stringArray)
 
 
-def _createQueryString(dct):
+def _createQueryStringBuf(dct: dict[str, str]) -> Array[c_char]:
     """Create a query string from a dictionary"""
     if not isinstance(dct, dict):
         raise TypeError("Query should be a dictionary")
-    return " ".join(
-        "%s:%s" % (k, v)
-        for k, v in dct.items()
-        if isinstance(k, str) and isinstance(v, str)
-    ).encode(getdefaultencoding(), errors=ENCODING_ERROR_HANDLER)
+    return create_string_buffer(
+        " ".join(
+            f"{k}:{v}"
+            for k, v in dct.items()
+            if isinstance(k, str) and isinstance(v, str)
+        ).encode(getdefaultencoding(), errors=ENCODING_ERROR_HANDLER)
+    )
 
 
 register(liblouis.lou_free)
 
 liblouis.lou_version.restype = c_char_p
+liblouis.lou_version.argtypes = ()
 
 liblouis.lou_charSize.restype = c_int
+liblouis.lou_charSize.argtypes = ()
 
+liblouis.lou_translateString.restype = c_int
 liblouis.lou_translateString.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -153,6 +179,7 @@ liblouis.lou_translateString.argtypes = (
     c_int,
 )
 
+liblouis.lou_translate.restype = c_int
 liblouis.lou_translate.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -167,6 +194,7 @@ liblouis.lou_translate.argtypes = (
     c_int,
 )
 
+liblouis.lou_backTranslateString.restype = c_int
 liblouis.lou_backTranslateString.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -178,6 +206,7 @@ liblouis.lou_backTranslateString.argtypes = (
     c_int,
 )
 
+liblouis.lou_backTranslate.restype = c_int
 liblouis.lou_backTranslate.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -192,6 +221,7 @@ liblouis.lou_backTranslate.argtypes = (
     c_int,
 )
 
+liblouis.lou_hyphenate.restype = c_int
 liblouis.lou_hyphenate.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -200,12 +230,16 @@ liblouis.lou_hyphenate.argtypes = (
     c_int,
 )
 
+liblouis.lou_checkTable.restype = c_int
 liblouis.lou_checkTable.argtypes = (c_char_p,)
 
+liblouis.lou_compileString.restype = c_int
 liblouis.lou_compileString.argtypes = (c_char_p, c_char_p)
 
+liblouis.lou_getTypeformForEmphClass.restype = c_ushort
 liblouis.lou_getTypeformForEmphClass.argtypes = (c_char_p, c_char_p)
 
+liblouis.lou_dotsToChar.restype = c_int
 liblouis.lou_dotsToChar.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -214,6 +248,7 @@ liblouis.lou_dotsToChar.argtypes = (
     c_int,
 )
 
+liblouis.lou_charToDots.restype = c_int
 liblouis.lou_charToDots.argtypes = (
     c_char_p,
     POINTER(c_char),
@@ -225,6 +260,7 @@ liblouis.lou_charToDots.argtypes = (
 LogCallback = _functype(None, c_int, c_char_p)
 
 liblouis.lou_registerLogCallback.restype = None
+liblouis.lou_registerLogCallback.argtypes = (LogCallback,)
 
 liblouis.lou_setLogLevel.restype = None
 liblouis.lou_setLogLevel.argtypes = (c_int,)
@@ -242,10 +278,7 @@ liblouis.lou_freeTableFiles.restype = None
 liblouis.lou_freeTableFiles.argtypes = (POINTER(c_char_p),)
 
 liblouis.lou_getTableInfo.restype = POINTER(c_char)
-liblouis.lou_getTableInfo.argtypes = (
-    c_char_p,
-    c_char_p,
-)
+liblouis.lou_getTableInfo.argtypes = (c_char_p, c_char_p)
 
 liblouis.lou_freeTableInfo.restype = None
 liblouis.lou_freeTableInfo.argtypes = (liblouis.lou_getTableInfo.restype,)
@@ -257,47 +290,45 @@ liblouis.lou_free.restype = None
 liblouis.lou_free.argtypes = ()
 
 
-def version():
+def version() -> str:
     """Obtain version information for liblouis.
-    @return: The version of liblouis, plus other information, such as
+    :return: The version of liblouis, plus other information, such as
         the release date and perhaps notable changes.
-    @rtype: str
     """
     return liblouis.lou_version().decode("ASCII")
 
 
-def charSize():
+def charSize() -> int:
     """Obtain charSize information for liblouis.
-    @return: The size of the widechar with which liblouis was compiled.
-    @rtype: int
+    :return: The size of the widechar with which liblouis was compiled.
     """
     return liblouis.lou_charSize()
 
 
-def translate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
+def translate(
+    tableList: TableListT,
+    inbuf: str,
+    typeform: list[int] | None = None,
+    cursorPos: int = 0,
+    mode: int = 0,
+) -> tuple[str, list[int], list[int], int]:
     """Translate a string of characters, providing position information.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inbuf: The string to translate.
-    @type inbuf: str
-    @param typeform: A list of typeform constants indicating the typeform for each position in inbuf,
-        C{None} for no typeform information.
-    @type typeform: list of int
-    @param cursorPos: The position of the cursor in inbuf.
-    @type cursorPos: int
-    @param mode: The translation mode; add multiple values for a combined mode.
-    @type mode: int
-    @return: A tuple of: the translated string,
+    :param tableList: A list of translation tables.
+    :param inbuf: The string to translate.
+    :param typeform: A list of typeform constants indicating the typeform for each position in inbuf,
+        `None` for no typeform information.
+    :param cursorPos: The position of the cursor in inbuf.
+    :param mode: The translation mode; add multiple values for a combined mode.
+    :return: A tuple of: the translated string,
         a list of input positions for each position in the output,
         a list of output positions for each position in the input, and
         the position of the cursor in the output.
-    @rtype: (str, list of int, list of int, int)
-    @raise RuntimeError: If a complete translation could not be done.
-    @see: lou_translate in the liblouis documentation
+    :raise RuntimeError: If a complete translation could not be done.
+    :see: lou_translate in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
-    inlen = c_int(len(inbuf) // wideCharBytes)
+    tableBuf: Array[c_char] = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
+    inlen = c_int(len(_inbuf) // wideCharBytes)
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
     typeformbuf = None
@@ -305,104 +336,104 @@ def translate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
         typeformbuf = _createTypeformbuf(outlen.value, typeform)
     inPos = (c_int * outlen.value)()
     outPos = (c_int * inlen.value)()
-    cursorPos = c_int(cursorPos)
+    _cursorPos = c_int(cursorPos)
     if not liblouis.lou_translate(
-        tablesString,
-        inbuf,
-        byref(inlen),
+        tableBuf,
+        _inbuf,
+        inlen,
         outbuf,
-        byref(outlen),
+        outlen,
         typeformbuf,
         None,
         outPos,
         inPos,
-        byref(cursorPos),
+        _cursorPos,
         mode,
     ):
         raise RuntimeError(
-            "Can't translate: tables %s, inbuf %s, typeform %s, cursorPos %s, mode %s"
-            % (tableList, inbuf, typeform, cursorPos, mode)
+            f"Can't translate: tables {tableList!r}, inbuf {inbuf!r}, typeform {typeform!r}, cursorPos {cursorPos!r}, mode {mode!r}"
         )
     if isinstance(typeform, list) and typeformbuf is not None:
         typeform[:] = list(typeformbuf)
     return (
         outbuf.raw[: outlen.value * wideCharBytes].decode(
-            conversionEncoding, errors=ENCODING_ERROR_HANDLER
+            conversionEncoding,
+            errors=ENCODING_ERROR_HANDLER,
         ),
         inPos[: outlen.value],
         outPos[: inlen.value],
-        cursorPos.value,
+        _cursorPos.value,
     )
 
 
-def translateString(tableList, inbuf, typeform=None, mode=0):
+def translateString(
+    tableList: TableListT,
+    inbuf: str,
+    typeform: list[int] | None = None,
+    mode: int = 0,
+) -> str:
     """Translate a string of characters.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inbuf: The string to translate.
-    @type inbuf: str
-    @param typeform: A list of typeform constants indicating the typeform for each position in inbuf,
-        C{None} for no typeform information.
-    @type typeform: list of int
-    @param mode: The translation mode; add multiple values for a combined mode.
-    @type mode: int
-    @return: The translated string.
-    @rtype: str
-    @raise RuntimeError: If a complete translation could not be done.
-    @see: lou_translateString in the liblouis documentation
+    :param tableList: A list of translation tables.
+    :param inbuf: The string to translate.
+    :param typeform: A list of typeform constants indicating the typeform for each position in inbuf,
+        `None` for no typeform information.
+    :param mode: The translation mode; add multiple values for a combined mode.
+    :return: The translated string.
+    :raise RuntimeError: If a complete translation could not be done.
+    :see: lou_translateString in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
-    inlen = c_int(len(inbuf) // wideCharBytes)
+    tableBuf = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
+    inlen = c_int(len(_inbuf) // wideCharBytes)
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
     typeformbuf = None
     if typeform:
         typeformbuf = _createTypeformbuf(outlen.value, typeform)
     if not liblouis.lou_translateString(
-        tablesString,
-        inbuf,
-        byref(inlen),
+        tableBuf,
+        _inbuf,
+        inlen,
         outbuf,
-        byref(outlen),
+        outlen,
         typeformbuf,
         None,
         mode,
     ):
         raise RuntimeError(
-            "Can't translate: tables %s, inbuf %s, typeform %s, mode %s"
-            % (tableList, inbuf, typeform, mode)
+            f"Can't translate: tables {tableList}, inbuf {inbuf}, typeform {typeform}, mode {mode}"
         )
     if isinstance(typeform, list) and typeformbuf is not None:
         typeform[:] = list(typeformbuf)
     return outbuf.raw[: outlen.value * wideCharBytes].decode(
-        conversionEncoding, errors=ENCODING_ERROR_HANDLER
+        conversionEncoding,
+        errors=ENCODING_ERROR_HANDLER,
     )
 
 
-def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
+def backTranslate(
+    tableList: TableListT,
+    inbuf: str,
+    typeform: list[int] | None = None,
+    cursorPos: int = 0,
+    mode: int = 0,
+) -> tuple[str, list[int], list[int], int]:
     """Back translates a string of characters, providing position information.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inbuf: Braille to back translate.
-    @type inbuf: str
-    @param typeform: List where typeform constants will be placed.
-    @type typeform: list
-    @param cursorPos: Position of cursor.
-    @type cursorPos: int
-    @param mode: Translation mode.
-    @type mode: int
-    @return: A tuple: A string of the back translation,
+    :param tableList: A list of translation tables.
+    :param inbuf: Braille to back translate.
+    :param typeform: List where typeform constants will be placed.
+    :param cursorPos: Position of cursor.
+    :param mode: Translation mode.
+    :return: A tuple: A string of the back translation,
         a list of input positions for each position in the output,
         a list of the output positions for each position in the input and
         the position of the cursor in the output.
-    @rtype: (str, list of int, list of int, int)
-    @raise RuntimeError: If a complete back translation could not be done.
-    @see: lou_backTranslate in the liblouis documentation.
+    :raise RuntimeError: If a complete back translation could not be done.
+    :see: lou_backTranslate in the liblouis documentation.
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
-    inlen = c_int(len(inbuf) // wideCharBytes)
+    tableBuf = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
+    inlen = c_int(len(_inbuf) // wideCharBytes)
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
     typeformbuf = None
@@ -410,212 +441,207 @@ def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
         typeformbuf = _createTypeformbuf(outlen.value)
     inPos = (c_int * outlen.value)()
     outPos = (c_int * inlen.value)()
-    cursorPos = c_int(cursorPos)
+    _cursorPos = c_int(cursorPos)
     if not liblouis.lou_backTranslate(
-        tablesString,
-        inbuf,
-        byref(inlen),
+        tableBuf,
+        _inbuf,
+        inlen,
         outbuf,
-        byref(outlen),
+        outlen,
         typeformbuf,
         None,
         outPos,
         inPos,
-        byref(cursorPos),
+        _cursorPos,
         mode,
     ):
         raise RuntimeError(
-            "Can't back translate: tables %s, inbuf %s, typeform %s, cursorPos %d, mode %d"
-            % (tableList, inbuf, typeform, cursorPos, mode)
+            f"Can't back translate: tables {tableList!r}, inbuf {inbuf!r}, typeform {typeform!r}, cursorPos {cursorPos!r}, mode {mode!r}"
         )
     if isinstance(typeform, list) and typeformbuf is not None:
         typeform[:] = list(typeformbuf)
     return (
         outbuf.raw[: outlen.value * wideCharBytes].decode(
-            conversionEncoding, errors=ENCODING_ERROR_HANDLER
+            conversionEncoding,
+            errors=ENCODING_ERROR_HANDLER,
         ),
         inPos[: outlen.value],
         outPos[: inlen.value],
-        cursorPos.value,
+        _cursorPos.value,
     )
 
 
-def backTranslateString(tableList, inbuf, typeform=None, mode=0):
+def backTranslateString(
+    tableList: TableListT,
+    inbuf: str,
+    typeform: list[int] | None = None,
+    mode: int = 0,
+) -> str:
     """Back translate from Braille.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inbuf: The Braille to back translate.
-    @type inbuf: str
-    @param typeform: List for typeform constants to be put in.
-        If you don't want typeform data then give None
-    @type typeform: list
-    @param mode: The translation mode
-    @type mode: int
-    @return: The back translation of inbuf.
-    @rtype: str
-    @raise RuntimeError: If a complete back translation could not be done.
-    @see: lou_backTranslateString in the liblouis documentation.
+    :param tableList: A list of translation tables.
+    :param inbuf: The Braille to back translate.
+    :param typeform: List for typeform constants to be put in.
+        If you don't want typeform data then give `None`
+    :param mode: The translation mode
+    :return: The back translation of inbuf.
+    :raise RuntimeError: If a complete back translation could not be done.
+    :see: lou_backTranslateString in the liblouis documentation.
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
-    inlen = c_int(len(inbuf) // wideCharBytes)
+    tableBuf = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
+    inlen = c_int(len(_inbuf) // wideCharBytes)
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_string_buffer(outlen.value * wideCharBytes)
     typeformbuf = None
     if isinstance(typeform, list):
         typeformbuf = _createTypeformbuf(outlen.value)
     if not liblouis.lou_backTranslateString(
-        tablesString,
-        inbuf,
-        byref(inlen),
+        tableBuf,
+        _inbuf,
+        inlen,
         outbuf,
-        byref(outlen),
+        outlen,
         typeformbuf,
         None,
         mode,
     ):
         raise RuntimeError(
-            "Can't back translate: tables %s, inbuf %s, mode %d"
-            % (tableList, inbuf, mode)
+            f"Can't back translate: tables {tableList!r}, inbuf {inbuf!r}, mode {mode!r}"
         )
     if isinstance(typeform, list) and typeformbuf is not None:
         typeform[:] = list(typeformbuf)
     return outbuf.raw[: outlen.value * wideCharBytes].decode(
-        conversionEncoding, errors=ENCODING_ERROR_HANDLER
+        conversionEncoding,
+        errors=ENCODING_ERROR_HANDLER,
     )
 
 
-def hyphenate(tableList, inbuf, mode=0):
+def hyphenate(tableList: TableListT, inbuf: str, mode: int = 0) -> str:
     """Get information for hyphenation.
-    @param tableList: A list of translation tables and hyphenation
+    :param tableList: A list of translation tables and hyphenation
         dictionaries.
-    @type tableList: list of str
-    @param inbuf: The text to get hyphenation information about.
+    :param inbuf: The text to get hyphenation information about.
         This should be a single word and leading/trailing whitespace
         and punctuation is ignored.
-    @type inbuf: str
-    @param mode: Lets liblouis know if inbuf is plain text or Braille.
+    :param mode: Lets liblouis know if inbuf is plain text or Braille.
         Set to 0 for text and anyother value for Braille.
-    @type mode: int
-    @return: A string with '1' at the beginning of every syllable
+    :return: A string with '1' at the beginning of every syllable
         and '0' elsewhere.
-    @rtype: str
-    @raise RuntimeError: If hyphenation data could not be produced.
-    @see: lou_hyphenate in the liblouis documentation.
+    :raise RuntimeError: If hyphenation data could not be produced.
+    :see: lou_hyphenate in the liblouis documentation.
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
-    inlen = c_int(len(inbuf) // wideCharBytes)
+    tableBuf = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
+    inlen = c_int(len(_inbuf) // wideCharBytes)
     hyphen_string = create_string_buffer(inlen.value + 1)
-    if not liblouis.lou_hyphenate(tablesString, inbuf, inlen, hyphen_string, mode):
+    if not liblouis.lou_hyphenate(
+        tableBuf,
+        _inbuf,
+        inlen,
+        hyphen_string,
+        mode,
+    ):
         raise RuntimeError(
-            "Can't hyphenate: tables %s, inbuf %s, mode %d" % (tableList, inbuf, mode)
+            f"Can't hyphenate: tables {tableList!r}, inbuf {inbuf!r}, mode {mode!r}"
         )
     return hyphen_string.value.decode("ASCII")
 
 
-def checkTable(tableList):
+def checkTable(tableList: TableListT) -> None:
     """Check if the specified tables can be found and compiled.
         This can be used to check if a list of tables contains errors
         before sending it to other liblouis functions
         that accept a list of tables.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @raise RuntimeError: If compilation failed.
-    @see: lou_checkTable in the liblouis documentation
+    :param tableList: A list of translation tables.
+    :raise RuntimeError: If compilation failed.
+    :see: lou_checkTable in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    if not liblouis.lou_checkTable(tablesString):
-        raise RuntimeError("Can't compile: tables %s" % tableList)
+    tableBuf = _createTableBuf(tableList)
+    if not liblouis.lou_checkTable(tableBuf):
+        raise RuntimeError(f"Can't compile: tables {tableList}")
 
 
-def compileString(tableList, inString):
+def compileString(tableList: TableListT, inString: str) -> None:
     """Compile a table entry on the fly at run-time.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inString: The table entry to be added.
-    @type inString: str
-    @raise RuntimeError: If compilation of the entry failed.
-    @see: lou_compileString in the liblouis documentation
+    :param tableList: A list of translation tables.
+    :param inString: The table entry to be added.
+    :raise RuntimeError: If compilation of the entry failed.
+    :see: lou_compileString in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    inBytes = inString.encode("ASCII") if isinstance(inString, str) else bytes(inString)
-    if not liblouis.lou_compileString(tablesString, inBytes):
+    tableBuf = _createTableBuf(tableList)
+    inbuf = create_string_buffer(
+        inString.encode("ASCII") if isinstance(inString, str) else bytes(inString)
+    )
+    if not liblouis.lou_compileString(tableBuf, inbuf):
         raise RuntimeError(
-            "Can't compile entry: tables %s, inString %s" % (tableList, inString)
+            f"Can't compile entry: tables {tableList}, inString {inString}"
         )
 
 
-def getTypeformForEmphClass(tableList, emphClass):
+def getTypeformForEmphClass(tableList: TableListT, emphClass: str) -> int:
     """Get the typeform bit for the named emphasis class.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param emphClass: An emphasis class name.
-    @type emphClass: str
-    @see: lou_getTypeformForEmphClass in the liblouis documentation
+    :param tableList: A list of translation tables.
+    :param emphClass: An emphasis class name.
+    :see: lou_getTypeformForEmphClass in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    emphClass = emphClass.encode("ASCII")
-    return liblouis.lou_getTypeformForEmphClass(tablesString, emphClass)
+    tableBuf = _createTableBuf(tableList)
+    bEmphClass = emphClass.encode("ASCII")
+    return liblouis.lou_getTypeformForEmphClass(tableBuf, bEmphClass)
 
 
-def dotsToChar(tableList, inbuf):
+def dotsToChar(tableList: TableListT, inbuf: str) -> str:
     """Convert a string of dot patterns to a string of characters according to the specifications in tableList.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inbuf: a string of dot patterns, either in liblouis format or Unicode braille.
-    @type inbuf: str
-    @raise RuntimeError: If a complete conversion could not be done.
-    @see: lou_dotsToChar in the liblouis documentation
+    :param tableList: A list of translation tables.
+    :param inbuf: a string of dot patterns, either in liblouis format or Unicode braille.
+    :raise RuntimeError: If a complete conversion could not be done.
+    :see: lou_dotsToChar in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
+    tableBuf = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
     length = c_int(len(inbuf) // wideCharBytes)
     outbuf = create_string_buffer(length.value * wideCharBytes)
-    if not liblouis.lou_dotsToChar(tablesString, inbuf, outbuf, length, 0):
+    if not liblouis.lou_dotsToChar(tableBuf, _inbuf, outbuf, length, 0):
         raise RuntimeError(
-            "Can't convert dots to char: tables %s, inbuf %s" % (tableList, inbuf)
+            f"Can't convert dots to char: tables {tableList}, inbuf {inbuf!r}"
         )
     return outbuf.raw[: length.value * wideCharBytes].decode(
-        conversionEncoding, errors=ENCODING_ERROR_HANDLER
+        conversionEncoding,
+        errors=ENCODING_ERROR_HANDLER,
     )
 
 
-def charToDots(tableList, inbuf, mode=0):
+def charToDots(tableList: TableListT, inbuf: str, mode: int = 0) -> str:
     """Convert a string of characterss to a string of dot patterns according to the specifications in tableList.
-    @param tableList: A list of translation tables.
-    @type tableList: list of str
-    @param inbuf: a string of characters.
-    @type inbuf: str
-    @param mode: The translation mode; add multiple values for a combined mode.
-    @type mode: int
-    @raise RuntimeError: If a complete conversion could not be done.
-    @see: lou_charToDots in the liblouis documentation
+    :param tableList: A list of translation tables.
+    :param inbuf: a string of characters.
+    :param mode: The translation mode; add multiple values for a combined mode.
+    :raise RuntimeError: If a complete conversion could not be done.
+    :see: lou_charToDots in the liblouis documentation
     """
-    tablesString = _createTablesString(tableList)
-    inbuf = createEncodedByteString(inbuf)
+    tableBuf = _createTableBuf(tableList)
+    _inbuf = _createEncodedBuf(inbuf)
     length = c_int(len(inbuf) // wideCharBytes)
     outbuf = create_string_buffer(length.value * wideCharBytes)
-    if not liblouis.lou_charToDots(tablesString, inbuf, outbuf, length, mode):
+    if not liblouis.lou_charToDots(tableBuf, _inbuf, outbuf, length, mode):
         raise RuntimeError(
-            "Can't convert char to dots: tables %s, inbuf %s, mode %d"
-            % (tableList, inbuf, mode)
+            f"Can't convert char to dots: tables {tableList!r}, inbuf {inbuf!r}, mode {mode!r}"
         )
     return outbuf.raw[: length.value * wideCharBytes].decode(
-        conversionEncoding, errors=ENCODING_ERROR_HANDLER
+        conversionEncoding,
+        errors=ENCODING_ERROR_HANDLER,
     )
 
 
-def registerLogCallback(logCallback):
+def registerLogCallback(logCallback) -> None:
     """Register logging callbacks.
-    Set to C{None} for default callback.
-    @param logCallback: The callback to use.
+    Set to `None` for default callback.
+    :param logCallback: The callback to use.
         The callback must take two arguments:
-        @param level: The log level on which a message is logged.
-        @type level: int
-        @param message: The logged message.
-            Note that the callback should provide its own ASCII decoding routine.
-        @type message: bytes
+        :param level: The log level on which a message is logged.
+        :type level: int
+        :param message: The logged message.
+            Note that the callback should provide its own UTF-8 decoding routine.
+        :type message: bytes
 
         Example callback:
 
@@ -623,36 +649,36 @@ def registerLogCallback(logCallback):
         def incomingLouisLog(level, message):
             print("Message %s logged at level %d" % (message.decode("ASCII"), level))
 
-    @type logCallback: L{LogCallback}
+    @type logCallback: `LogCallback`
     """
-    if logCallback is not None and not isinstance(logCallback, LogCallback):
+    if logCallback is None:
+        logCallback = LogCallback(0)
+    elif not isinstance(logCallback, LogCallback):
         raise TypeError(
-            "logCallback should be of type {} or NoneType".format(LogCallback.__name__)
+            f"logCallback should be of type {LogCallback.__name__} or NoneType"
         )
     return liblouis.lou_registerLogCallback(logCallback)
 
 
-def setLogLevel(level):
+def setLogLevel(level: int) -> None:
     """Set the level for logging callback to be called at.
-    @param level: one of the C{LOG_*} constants.
-    @type level: int
-    @raise ValueError: If an invalid log level is provided.
+    :param level: one of the C{LOG_*} constants.
+    :raise ValueError: If an invalid log level is provided.
     """
     if level not in logLevels:
         raise ValueError("Level %d is an invalid log level" % level)
     return liblouis.lou_setLogLevel(level)
 
 
-def findTable(query):
+def findTable(query: dict[str, str]) -> str | None:
     """Find the best match for a query
 
     Returns the name of the table, or None when no match can be
     found. An error message is given when the query is invalid. Freeing.
-    @param query: The query to lookup.
-    @type query: dict[str, str]
+    :param query: The query to lookup.
     """
-    query = _createQueryString(query)
-    result = liblouis.lou_findTable(query)
+    queryStr = _createQueryStringBuf(query)
+    result = liblouis.lou_findTable(queryStr)
     if not result:
         return None
     string = string_at(result).decode(fileSystemEncoding, errors=ENCODING_ERROR_HANDLER)
@@ -660,28 +686,27 @@ def findTable(query):
     return string
 
 
-def findTables(query):
+def findTables(query: dict[str, str]) -> list[str]:
     """Find all matches for a query, best match first.
 
     Returns the names of the matched tables.
     An error message is given when the query is invalid.
-    @param query: The query to lookup.
-    @type query: dict[str, str]
+    :param query: The query to lookup.
     """
-    query = _createQueryString(query)
-    result = liblouis.lou_findTables(query)
+    queryStr = _createQueryStringBuf(query)
+    result = liblouis.lou_findTables(queryStr)
     return list(_processTableFiles(result))
 
 
-def getTableInfo(tableName, key):
+def getTableInfo(tableName: str, key: str) -> str | None:
     """Read metadata from a file.
 
     Returns the value of the first occurring metadata field specified by
     `key' in `table', or None when the field does not exist.
     """
-    tableName = createEncodedByteString(tableName, fileSystemEncoding)
-    key = createEncodedByteString(key, getdefaultencoding())
-    result = liblouis.lou_getTableInfo(tableName, key)
+    bTableName = _createEncodedBuf(tableName, fileSystemEncoding)
+    bKey = _createEncodedBuf(key, getdefaultencoding())
+    result = liblouis.lou_getTableInfo(bTableName, bKey)
     if not result:
         return None
     string = string_at(result).decode(errors=ENCODING_ERROR_HANDLER)
@@ -689,7 +714,7 @@ def getTableInfo(tableName, key):
     return string
 
 
-def listTables():
+def listTables() -> list[str]:
     """List available tables.
 
     Returns the names of available tables as list  of strings.
